# -*- coding: utf-8 -*-
"""
Created on 07.12.2018

@author: engler/seyler
"""

from IPMrc import *
import numpy as np
import matplotlib.pyplot as plt
import statistics
import matplotlib.patches as mpatches
from matplotlib import rcParams

def to_integer_repre(numberstr):
    parts = numberstr.split("e")
    return round(float(parts[0]) * 10 ** int(parts[1][1:]), 4)

def messungen_from_txt(filename, nr_meas):
    messungen = []
    with open(filename) as file:
        lines = file.read().splitlines()
        for i in range(0, len(lines) // (3 + (nr_meas*2))):
            meas = lines[i * (3 + (nr_meas*2)): i * (3 + (nr_meas*2)) + (3 + (nr_meas*2))]
            param = (meas[0][0], meas[0][2])
            complete_raw = []
            transfer_raw = []
            bitrate_raw = []
            for l in range(0, nr_meas):
                numb = meas[2 + l].split("Info: Total request time: ")
                complete_raw.append(numb[1].split(" ")[0])
                trans = numb[1].split(";")
                trans2 = trans[1].split(" Raw transfer time: ")
                transfer_raw.append(trans2[1].split(" ")[0])
                rate2 = trans[2].split(" Request transfer rate: ")
                bitrate_raw.append(to_integer_repre(rate2[1].split(" ")[0]))

            complete_ver = []
            transfer_ver = []
            bitrate_ver = []
            for l in range(0, nr_meas):
                numb = meas[nr_meas+3 + l].split("Info: Total request time: ")
                complete_ver.append(numb[1].split(" ")[0])
                trans = numb[1].split(";")
                trans2 = trans[1].split(" Raw transfer time: ")
                transfer_ver.append(trans2[1].split(" ")[0])
                rate2 = trans[2].split(" Request transfer rate: ")
                bitrate_ver.append(to_integer_repre(rate2[1].split(" ")[0]))

            messungen.append(
                (param[0], param[1], complete_raw, transfer_raw, bitrate_raw, complete_ver, transfer_ver, bitrate_ver))
    return messungen


def plot_graph(mes, savename, anzahl, labeladd):

    bars_comp_raw = [statistics.mean([int(h) for h in x[2]]) / 1000 for x in mes]
    bars_comp_raw_std = [statistics.stdev([int(h) for h in x[2]]) / 1000 for x in mes]

    bars_trans_raw = [statistics.mean([int(h) for h in x[3]]) / 1000 for x in mes]

    bars_diff_raw = [x - y for x in bars_comp_raw for y in bars_trans_raw][::anzahl+1]

    bars_comp_ver = [statistics.mean([int(h) for h in x[5]]) / 1000 for x in mes]
    bars_comp_ver_std = [statistics.stdev([int(h) for h in x[5]]) / 1000 for x in mes]

    bars_trans_ver = [statistics.mean([int(h) for h in x[6]]) / 1000 for x in mes]

    bars_diff_ver = [x - y for x in bars_comp_ver for y in bars_trans_ver][::anzahl+1]

    barWidth = 0.2

    r = np.arange(0, anzahl)
    r1 = [x - barWidth/2 for x in r]
    r2 = [x + barWidth/2 for x in r]

    plt.figure()

    if len(labeladd) == 2:
        names = [x[0] + " Laser, " + x[1] + " Steps\n" for x in mes]
        for index, ob in enumerate(names):
            names[index] = names[index] + labeladd[index //2]
    else:
        names = [x[0] + " Laser,\n" + x[1] + " Steps" for x in mes]

    # Lower part
    plt.bar(r1, bars_trans_raw, color=fhg_gruen, width=barWidth)
    plt.bar(r2, bars_diff_ver, color=fhg_orange, width=barWidth)

    # Upper part
    plt.bar(r1, bars_diff_raw, yerr=bars_comp_raw_std, capsize=10, bottom=bars_trans_raw, color=fhg_blau,
            width=barWidth)
    plt.bar(r2, bars_trans_ver, yerr=bars_comp_ver_std, capsize=10, bottom=bars_diff_ver, color=fhg_rot,
            width=barWidth)

    plt.xticks(r, names)
    plt.ylabel("Time $t$ / s")

    patches = []
    patches.append(mpatches.Patch(color=fhg_gruen, label='Transfer raw data'))
    patches.append(mpatches.Patch(color=fhg_blau, label='Processing external PC'))
    patches.append(mpatches.Patch(color=fhg_orange, label='Processing embedded system'))
    patches.append(mpatches.Patch(color=fhg_rot, label='Transfer processed data'))

    lgd = plt.legend(handles=patches, bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
                     ncol=2, mode="expand", borderaxespad=0., fontsize=10)

    plt.tight_layout()
    plt.savefig(savename, bbox_extra_artists=(lgd,), bbox_inches='tight',dpi=400)
    plt.show()

if __name__ == '__main__':
# Paths
	messdaten_pfad = r"."
	ergebnis_pfad = r"."
	ausgangsdateiname = r"\eval_transfer.png"
	mes = messungen_from_txt(messdaten_pfad + r"\data_transfer.txt", 10)
	plot_graph(mes, ergebnis_pfad + ausgangsdateiname, len(mes), [])
	print("File written to: " + ergebnis_pfad + ausgangsdateiname)
